//
//  ContentView.swift
//  Penny
//
//  Created by Katie DiPaola on 10/27/24.
//

import SwiftUI
import Combine

struct ContentView: View {

    // Note: @State means that a variable will listen for changes and
    // change within the view search results
    @State private var books: [Book] = []
    // input text
    @State var searchText: String = ""
    // timer
    @State private var cancellable: AnyCancellable?


    var body: some View {
        VStack {
            // header
            Text("Welcome to Penny IOS")
                .font(.system(size: 16))
                .padding(.vertical, 20)
            Text("Author Search")
                .font(.system(size: 32))
                .padding(.bottom, 10)
            Text("Please enter an author name:")

            // searchBar view from SearchBar.swift
            SearchBar(text: $searchText)
                .padding(.horizontal, 20)
                .padding(.bottom, 20)
                // when text is entered (aka input changes), delay and
                // then fetch from database
                .onChange(of: searchText) { newValue in
                    // Cancel the previous timer if it exists
                    cancellable?.cancel()
                    // Start a new timer for delay
                    cancellable = Just(newValue)
                        .delay(for: .milliseconds(500),
                            scheduler: RunLoop.main)
                        // fetch books from database
                        .sink { text in
                            let database = Database()
                            // handle completion (aka await return from
                            // fetchBooks)
                            database.fetchBooks(author: text) {
                                fetched_books in
                                self.books = fetched_books
                            }
                        }
                }
                // display result of current search
                NavigationView {
                    List(books, id: \.self) { book in
                        VStack (alignment: .leading){
                            Text(book.isbn + ": " + book.title)
                            Text(book.author)
                                .bold()
                        }
                    }
                    .background(Color(.systemGray6))
                }
        }
    }
}

#Preview {
    ContentView()
}
