//
//  Database.swift
//  Penny
//
//  Created by Katie DiPaola on 10/27/24.
//

import Foundation

class Database {

    let url = "https://pennyjson.onrender.com/searchresults?author="

    // get books where author starts with text in author
    // completion is used for async functions to notify when the
    // function returns with data
    func fetchBooks(author: String,
        completion: @escaping ([Book]) -> ()) {
        // exit function if author is empty
        if author.isEmpty {
            completion([])
            return
        }

        // add author to database url
        let queryUrl = URL(string: url + "\(author)")!
        // make asynchronous http request
        URLSession.shared.dataTask(with: queryUrl) { data,_,error in

            if let error = error {
                print("Error fetching books: \(error)")
                completion([])
                return
            }

            // guard lets us check if an optional value exists without
            // throwing an error
            guard let data = data
            else {
                completion([])
                return
            }

            // otherwise handle json data
            do {
                // decode into list of Book objects (defined in
                // Book.swift)
                let books = try JSONDecoder().decode(
                    [Book].self, from: data)
                // completion on the main thread (UI updates must occur
                // on main thread)
                DispatchQueue.main.async {
                    completion(books)
                }
            } catch {
                print("Error decoding JSON response: \(error)")
            }
        // start task (send http request we defined above)
        }.resume()
    }
}
