//
//  SearchBar.swift
//  Penny
//
//  Created by Katie DiPaola on 10/27/24.
//

import SwiftUI

// re-used this code from my app
struct SearchBar: View {

    // Note: @Binding means that the source of this variable is
    // elsewhere - it's a parameter of the view

    // input text
    @Binding var text: String
    // when input is being edited
    @State private var isEditing = false

    var body: some View {
        HStack {
            // input text field
            TextField("Search ...", text: $text)
                .padding(7)
                .padding(.horizontal, 25)
                .background(Color(.systemGray6))
                .cornerRadius(8)
                // make it look pretty
                .overlay(
                    HStack {
                        // uses a build-in IOS image
                        Image(systemName: "magnifyingglass")
                            .foregroundColor(.gray)
                            .frame(minWidth: 0, maxWidth: .infinity,
                               alignment: .leading)
                            .padding(.leading, 8)

                        // provide functionality to delete full user
                        // input
                        if isEditing {
                            Button(action: {
                                self.text = ""

                            }) {
                                // uses a build-in IOS image
                                Image(systemName:
                                    "multiply.circle.fill")
                                    .foregroundColor(.gray)
                                    .padding(.trailing, 8)
                            }
                        }
                    }
                )
                .padding(.horizontal, 10)
                // track when user is editing input
                .onTapGesture {
                    self.isEditing = true
                }
        }
    }
}

struct SearchBar_Previews: PreviewProvider {
    static var previews: some View {
        // view paramter (@Binding variable from above!)
        SearchBar(text: .constant(""))
    }
}
