#!/usr/bin/env python

#-----------------------------------------------------------------------
# pennydesktop1base.py
# Author: Bob Dondero
#-----------------------------------------------------------------------

import sys
import urllib.parse
import urllib.request
import json
import PyQt5.QtWidgets

#-----------------------------------------------------------------------

def create_widgets():

    author_label = PyQt5.QtWidgets.QLabel('Author: ')
    author_lineedit = PyQt5.QtWidgets.QLineEdit()
    submit_button = PyQt5.QtWidgets.QPushButton('Submit')
    books_textedit = PyQt5.QtWidgets.QTextEdit()
    books_textedit.setReadOnly(True)

    layout = PyQt5.QtWidgets.QGridLayout()
    layout.addWidget(author_label, 0, 0)
    layout.addWidget(author_lineedit, 0, 1)
    layout.addWidget(submit_button, 0, 2)
    layout.addWidget(books_textedit, 1, 0, 1, 3)
    layout.setRowStretch(0, 0)
    layout.setRowStretch(1, 1)
    layout.setColumnStretch(0, 0)
    layout.setColumnStretch(1, 1)
    layout.setColumnStretch(2, 0)

    frame = PyQt5.QtWidgets.QFrame()
    frame.setLayout(layout)

    window = PyQt5.QtWidgets.QMainWindow()
    window.setWindowTitle('Penny: Author Search')
    window.setCentralWidget(frame)
    screen_size = PyQt5.QtWidgets.QDesktopWidget().screenGeometry()
    window.resize(screen_size.width()//2, screen_size.height()//2)

    return (window, author_lineedit, submit_button, books_textedit)

#-----------------------------------------------------------------------

def author_slot_helper(server_url, author_lineedit, books_textedit):

    author = author_lineedit.text()
    encoded_author = urllib.parse.quote_plus(author)
    url = server_url + '/searchresults?author=' + encoded_author

    books_textedit.clear()

    try:
        with urllib.request.urlopen(url) as in_flo:
            response = in_flo.read()
            json_doc = response.decode('utf-8')
            books = json.loads(json_doc)

            if len(books) == 0:
                books_textedit.insertPlainText('(None)')
            else:
                pattern = '%s: <strong>%s</strong>: %s<br>'
                for book in books:
                    books_textedit.insertHtml(pattern %
                        (book['isbn'], book['author'], book['title']))

    except Exception as ex:
        books_textedit.insertPlainText(str(ex))

    books_textedit.repaint()  # Required on Mac.

#-----------------------------------------------------------------------

def main():

    if len(sys.argv) != 2:
        print('Usage: penny serverURL', file=sys.stderr)
        sys.exit(1)

    server_url = sys.argv[1]

    # Create and lay out the widgets.

    app = PyQt5.QtWidgets.QApplication(sys.argv)
    window, author_lineedit, submit_button, books_textedit = (
        create_widgets())

    # Handle signals.

    def author_slot():
        author_slot_helper(server_url, author_lineedit, books_textedit)
    submit_button.clicked.connect(author_slot)

    # Start up.

    window.show()
    author_slot()  # Populate books_textedit initially.
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()
